package java.beans;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Vector;

/*
 * Java core library component.
 *
 * Copyright (c) 1997, 1998
 *      Transvirtual Technologies, Inc.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file.
 */
public class PropertyChangeSupport
  implements Serializable
{
	private static final long serialVersionUID = 6401253773779951803L;
	private final HashMap PROPERTY_LISTENERS = new HashMap();
	private Object source;

public PropertyChangeSupport(Object sourceBean) {
	source = sourceBean;
}

public synchronized void addPropertyChangeListener(String property, PropertyChangeListener listener) {
	if (!PROPERTY_LISTENERS.containsKey(property)) {
			PROPERTY_LISTENERS.put(property, new Vector());
	}
	
	((Vector) PROPERTY_LISTENERS.get(property)).addElement(listener);
}

public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
	addPropertyChangeListener(null, listener);
}

public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {
	if ((oldValue == newValue) ||
	    (!PROPERTY_LISTENERS.containsKey(null) && !PROPERTY_LISTENERS.containsKey(propertyName))) {
		return;
	}

	firePropertyChange (propertyName, new PropertyChangeEvent (source, propertyName,
								   new Boolean (oldValue),
								   new Boolean (newValue)));
}

public void firePropertyChange(String propertyName, int oldValue, int newValue) {
        if ((oldValue == newValue) ||
            (!PROPERTY_LISTENERS.containsKey(null) && !PROPERTY_LISTENERS.containsKey(propertyName))) {
                return;
        }
            
        firePropertyChange (propertyName, new PropertyChangeEvent (source, propertyName,
                                                                   new Integer (oldValue),
                                                                   new Integer (newValue)));
}

public void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
	if ((oldValue!=null && oldValue.equals(newValue)) ||  
	    (!PROPERTY_LISTENERS.containsKey(null) && !PROPERTY_LISTENERS.containsKey(propertyName))) {
		return;
	}

	firePropertyChange (propertyName, new PropertyChangeEvent (source, propertyName,
								   oldValue, newValue));
}

public void firePropertyChange (String propertyName, PropertyChangeEvent pce) {
	// Is anyone interested ?
	Vector vec = (Vector) PROPERTY_LISTENERS.get (propertyName);
        if (vec == null) {
        	vec = (Vector) PROPERTY_LISTENERS.get (null);
        } else {
		vec = ((Vector)vec.clone());
                vec.addAll((Vector) PROPERTY_LISTENERS.get(null));
	}

        if (vec==null) {
		return;
	}

	int size;
	synchronized (this) {
		size = vec.size();
		if (size == 0) {
			return;
		}
	}
	for (int dis = 0; dis < size; dis++) {
		((PropertyChangeListener)vec.elementAt(dis)).propertyChange(pce);
	}
}

public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
	removePropertyChangeListener(null, listener);
}

public synchronized void removePropertyChangeListener(String property, PropertyChangeListener listener) {
	if (PROPERTY_LISTENERS.containsKey(property)) {
		((Vector) PROPERTY_LISTENERS.get(property)).removeElement(listener);
	}
}
}
